#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libn64.h>
#include <malloc.h>
#include "audio.h"

/*
 * alt-libn64 demo
 * by hcs
 */

#define XRES 320
#define YRES 250
unsigned short screen[XRES*YRES];

// execution begins here
int UltraMain(void) {
    struct VI_config video;
    int draw=1;

    // buffer for rendering text strings (only sprintf works at the moment)
    char buf[1024]; 

    // enable MI interrupts (on the CPU), but clear all specific device
    // interrupts
    set_MI_interrupt(1,1);

    // setup video
    video.bitdepth=16;
    video.xres=XRES;
    video.yres=YRES;
    video.xscale=0.5;
    video.yscale=0.25;

    video.framebuffer=UncachedAddr(screen);

    VI_FillScreen(&video,0);

    // commit video
    VI_WriteRegs(&video);

    //
    audio_init();

    while (1) {
        short * sb;
        int sbsamples;
        static int row=3;

        struct controller_data cd;
        static int lastx = -128;
        static int lasty = 0;
        unsigned long starttime,endtime;
        unsigned long audiostarttime,audioendtime;

        static unsigned long totaltime = 1;
        static unsigned long audiotime = 0;

        starttime = read_count();

        controller_Read(&cd);

        // get audio properties
        sb=audio_getbuf();
        sbsamples=audio_getbufsamplecount();

        short firstsample;
        static double d=0.0;
        static double f=0.0;

        audiostarttime = read_count();
        // generate audio
        for (int c=0;c<sbsamples;c++) {
            double progress = (double)c/sbsamples;
            sb[c*2+1]=sb[c*2]=sin(d)*10000*
                (1.0-
                 abs(cd.c[0].x*progress +
                  lastx*(1.0-progress))/128.0);

            d+=M_TWOPI/AUDIO_FREQUENCY*440.0*pow(2,
                    sin(f-M_PI_2)*2.0);
            /* continuous 
            d+=M_TWOPI/AUDIO_FREQUENCY*440.0*pow(2,
                    (cd.c[0].y*progress +
                     lasty*(1.0-progress))/128.0*2.0);
                     */

            /* discrete 
            d+=M_TWOPI/32000.0*440.0*pow(2,
                    (int)((cd.c[0].y*progress +
                           lasty*(1.0-progress))/128.0*16.0) / 8.0);
                           */
            while (d>M_TWOPI) d-=M_TWOPI;

            f+=M_TWOPI/AUDIO_FREQUENCY/2;

            while (f>M_TWOPI) f-=M_TWOPI;

            if (cd.c[0].start) {
                sb[c*2+1]=sb[c*2]=0;//10000;
                d = 0.0;
                f = 0.0;
            }
        }
        firstsample = sb[0];
        
        audioendtime = read_count();

        // queue for playback
        audio_done();

        if (draw) {
            /* tenths of milliseconds */
            unsigned long atime = audiotime*10000ULL/COUNTS_PER_SECOND;
            unsigned long ttime = totaltime*10000ULL/COUNTS_PER_SECOND;
            unsigned long btime = AUDIO_BUFFER_SAMPLES*10000ULL/AUDIO_FREQUENCY;
            sprintf(buf,"a%2lu.%lums t%2lu.%lums b%2lu.%lums d%d r%d",
                    atime/10,atime%10,ttime/10,ttime%10,btime/10,btime%10,
                    audio_buffers_deferred,audio_restarts);
            VI_DrawText(&video,16,row++*8,buf);
        }

        if (row > 27) {
            if (cd.c[0].A) {
                draw=0;
            } else {
                draw=1;
                row = 3;
                int i;
                int redrawcount;
                if (cd.c[0].B) redrawcount = 3;
                else redrawcount = 2;
                for (i=0;i<redrawcount;i++)
                    VI_FillScreen(&video,0);
            }
        }

        lastx = cd.c[0].x;
        lasty = cd.c[0].y;

        endtime = read_count();

        totaltime = endtime-starttime;
        audiotime = audioendtime-audiostarttime;
    }

    return 0;
}

